@@ -25,6 +25,7 @@
 #include "qgis_sip.h"
 #include "qgis_gui.h"
 #include "qgscoordinatereferencesystem.h"
+#include "qgslayertreeregistrybridge.h"
 
 class QAction;
 class QDialog;
@@ -42,6 +43,7 @@ class QgsLayoutCustomDropHandler;
 class QgsFeature;
 class QgsLayerTreeMapCanvasBridge;
 class QgsLayerTreeView;
+class QgsLayerTreeGroup;
 class QgsLayout;
 class QgsMasterLayoutInterface;
 class QgsLayoutDesignerInterface;
@@ -58,7 +60,7 @@ class QgsOptionsWidgetFactory;
 class QgsLocatorFilter;
 class QgsStatusBar;
 class QgsMeshLayer;
-class QgsBrowserModel;
+class QgsBrowserGuiModel;
 
 
 /**
@@ -234,6 +236,16 @@ class GUI_EXPORT QgisInterface : public QObject
      */
     virtual QMenu *pluginMenu() = 0;
 
+    /**
+     * Returns a reference to the main window "Plugin Help" sub-menu.
+     *
+     * Plugins are encouraged to insert help and about actions in this submenu instead of creating
+     * a submenu under the pluginMenu() which solely contains Plugin Help or About actions.
+     *
+     * \since QGIS 3.10
+     */
+    virtual QMenu *pluginHelpMenu() = 0;
+
     /**
      * Returns a reference to the main window "Raster" menu.
      */
@@ -522,6 +534,40 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual QAction *actionCheckQgisVersion() = 0;
     virtual QAction *actionAbout() = 0;
 
+    // Shape digitize actions
+    //! Returns the native add circle from 2 points action. Call trigger() on it to set the map tool.
+    virtual QAction *actionCircle2Points()  = 0;
+    //! Returns the native add circle from 3 points action. Call trigger() on it to set the map tool.
+    virtual QAction *actionCircle3Points()  = 0;
+    //! Returns the native add circle from 3 tangents action. Call trigger() on it to set the map tool.
+    virtual QAction *actionCircle3Tangents()  = 0;
+    //! Returns the native add circle from 2 tangents and a point action. Call trigger() on it to set the map tool.
+    virtual QAction *actionCircle2TangentsPoint()  = 0;
+    //! Returns the native add circle from center action. Call trigger() on it to set the map tool.
+    virtual QAction *actionCircleCenterPoint()  = 0;
+    //! Returns the native add ellipse from center and 2 points action. Call trigger() on it to set the map tool.
+    virtual QAction *actionEllipseCenter2Points()  = 0;
+    //! Returns the native add ellipse from center and a point action. Call trigger() on it to set the map tool.
+    virtual QAction *actionEllipseCenterPoint()  = 0;
+    //! Returns the native add ellipse from an extent action. Call trigger() on it to set the map tool.
+    virtual QAction *actionEllipseExtent()  = 0;
+    //! Returns the native add ellipse from foci action. Call trigger() on it to set the map tool.
+    virtual QAction *actionEllipseFoci()  = 0;
+    //! Returns the native add rectangle from center and a point action. Call trigger() on it to set the map tool.
+    virtual QAction *actionRectangleCenterPoint()  = 0;
+    //! Returns the native add rectangle from extent action. Call trigger() on it to set the map tool.
+    virtual QAction *actionRectangleExtent()  = 0;
+    //! Returns the native add rectangle from 3 points (distance from 2nd and 3rd points) action. Call trigger() on it to set the map tool.
+    virtual QAction *actionRectangle3PointsDistance()  = 0;
+    //! Returns the native add rectangle from 3 points (distance from projected 3rd point on segment p1 and p2) action. Call trigger() on it to set the map tool.
+    virtual QAction *actionRectangle3PointsProjected()  = 0;
+    //! Returns the native add regular polygon from 2 points action. Call trigger() on it to set the map tool.
+    virtual QAction *actionRegularPolygon2Points()  = 0;
+    //! Returns the native add regular polygon from center and a point action. Call trigger() on it to set the map tool.
+    virtual QAction *actionRegularPolygonCenterPoint()  = 0;
+    //! Returns the native add regular polygon from center and a corner action. Call trigger() on it to set the map tool.
+    virtual QAction *actionRegularPolygonCenterCorner()  = 0;
+
     /**
      * Access the vector layer tools instance.
      * With the help of this you can access methods like addFeature, startEditing
@@ -551,6 +597,13 @@ class GUI_EXPORT QgisInterface : public QObject
      */
     virtual void takeAppScreenShots( const QString &saveDirectory, const int categories = 0 ) {Q_UNUSED( saveDirectory ) Q_UNUSED( categories );}
 
+    /**
+     * Returns the insertion point.
+     * This represents the current layer tree group and index where newly added map layers should be inserted into.
+     * \since QGIS 3.10
+     */
+    virtual QgsLayerTreeRegistryBridge::InsertionPoint layerTreeInsertionPoint() = 0;
+
   public slots: // TODO: do these functions really need to be slots?
 
     /* Exposed functions */
@@ -597,8 +650,18 @@ class GUI_EXPORT QgisInterface : public QObject
 
     //! Adds (opens) a project
     virtual bool addProject( const QString &project ) = 0;
-    //! Starts a new blank project
-    virtual void newProject( bool promptToSaveFlag = false ) = 0;
+
+    /**
+     * Starts a new blank project.
+     *
+     * If \a promptToSaveFlag is TRUE then users will be prompted to save any currently open
+     * project (if that project has changes). If the flag is FALSE, then the current project will
+     * be closed without prompting to save (possibly resulting in data loss).
+     *
+     * Since QGIS 3.10.1, returns TRUE if a new project was created, or FALSE if the operation was not successful (e.g.
+     * the user opted to cancel when prompted to save the current project).
+     */
+    virtual bool newProject( bool promptToSaveFlag = false ) = 0;
 
     /**
      * Triggered when connections have changed.
@@ -950,6 +1013,14 @@ class GUI_EXPORT QgisInterface : public QObject
      */
     virtual void preloadForm( const QString &uifile ) = 0;
 
+    /**
+     * This will perform a search in the locator bar
+     * by setting the line edit text to \a searchText
+     * and automatically displaying any results.
+     * \since QGIS 3.10
+     */
+    virtual void locatorSearch( const QString &searchText ) = 0;
+
     /**
      * Registers a locator \a filter for the app's locator bar. Ownership of the filter is transferred to the
      * locator.
@@ -994,7 +1065,7 @@ class GUI_EXPORT QgisInterface : public QObject
      * creating a new browser model for every use.
      * \since QGIS 3.4
      */
-    virtual QgsBrowserModel *browserModel() = 0;
+    virtual QgsBrowserGuiModel *browserModel() = 0;
 
   signals:
 
