@@ -55,6 +55,9 @@ class CORE_EXPORT QgsFieldModel : public QAbstractItemModel
       FieldTypeRole = Qt::UserRole + 6, //!< Return the field type (if a field, return QVariant if expression)
       FieldOriginRole = Qt::UserRole + 7, //!< Return the field origin (if a field, returns QVariant if expression)
       IsEmptyRole = Qt::UserRole + 8, //!< Return if the index corresponds to the empty value
+      EditorWidgetType = Qt::UserRole + 9, //!< Editor widget type
+      JoinedFieldIsEditable = Qt::UserRole + 10, //!< TRUE if a joined field is editable (returns QVariant if not a joined field)
+      FieldIsWidgetEditable = Qt::UserRole + 11, //!< TRUE if a is editable from the widget
     };
 
     /**
@@ -75,7 +78,7 @@ class CORE_EXPORT QgsFieldModel : public QAbstractItemModel
     void setAllowExpression( bool allowExpression );
 
     /**
-     * Returns true if the model allows custom expressions to be created and displayed.
+     * Returns TRUE if the model allows custom expressions to be created and displayed.
      * \see setAllowExpression()
      */
     bool allowExpression() { return mAllowExpression; }
@@ -88,14 +91,14 @@ class CORE_EXPORT QgsFieldModel : public QAbstractItemModel
     void setAllowEmptyFieldName( bool allowEmpty );
 
     /**
-     * Returns true if the model allows the empty field ("not set") choice.
+     * Returns TRUE if the model allows the empty field ("not set") choice.
      * \see setAllowEmptyFieldName()
      * \since QGIS 3.0
      */
     bool allowEmptyFieldName() const { return mAllowEmpty; }
 
     /**
-     * Returns true if a string represents a field reference, or false if it is an
+     * Returns TRUE if a string represents a field reference, or FALSE if it is an
      * expression consisting of more than direct field reference.
      */
     bool isField( const QString &expression ) const;
@@ -135,6 +138,28 @@ class CORE_EXPORT QgsFieldModel : public QAbstractItemModel
      */
     static QString fieldToolTip( const QgsField &field );
 
+    /**
+     * Manually sets the \a fields to use for the model.
+     *
+     * This method should only be used when the model ISN'T associated with a layer()
+     * and needs to show the fields from an arbitrary field collection instead. Calling
+     * setFields() will automatically clear any existing layer().
+     *
+     * \see fields()
+     * \since QGIS 3.14
+     */
+    void setFields( const QgsFields &fields );
+
+    /**
+     * Returns the fields currently shown in the model.
+     *
+     * This will either be fields from the associated layer() or the fields
+     * manually set by a call to setFields().
+     *
+     * \since QGIS 3.14
+     */
+    QgsFields fields() const;
+
   public slots:
 
     /**
