@@ -25,6 +25,7 @@
 #include "qgsrectangle.h"
 #include "qgsfeatureid.h"
 #include "qgsgeometry.h"
+#include "qgscustomdrophandler.h"
 
 #include <QDomDocument>
 #include <QGraphicsView>
@@ -64,6 +65,7 @@ class QgsMapTool;
 class QgsSnappingUtils;
 class QgsRubberBand;
 class QgsMapCanvasAnnotationItem;
+class QgsReferencedRectangle;
 
 /**
  * \ingroup gui
@@ -205,9 +207,29 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Returns the combined extent for all layers on the map canvas
     QgsRectangle fullExtent() const;
 
-    //! Sets the extent of the map canvas
+    /**
+     * Sets the extent of the map canvas to the specified rectangle.
+     *
+     * The \a magnified argument dictates whether existing canvas constraints such
+     * as a scale lock should be respected or not during the operation. If \a magnified is
+     * TRUE then an existing scale lock constraint will be applied. This means that the final
+     * visible canvas extent may not match the specified extent.
+     *
+     * If \a magnified is FALSE then scale lock settings will be ignored, and the specified
+     * rectangle will ALWAYS be visible in the canvas.
+     */
     void setExtent( const QgsRectangle &r, bool magnified = false );
 
+    /**
+     * Sets the canvas to the specified \a extent.
+     *
+     * \returns TRUE if the zoom was successful.
+     *
+     * \throws QgsCsException if a transformation error occurred.
+     * \since QGIS 3.10
+     */
+    bool setReferencedExtent( const QgsReferencedRectangle &extent ) SIP_THROW( QgsCsException );
+
     /**
      * Gets the current map canvas rotation in clockwise degrees
      * \since QGIS 2.8
@@ -439,14 +461,20 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     /**
      * Zooms the canvas to a specific \a scale.
      * The scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
+
+     * If \a ignoreScaleLock is set to TRUE, then any existing constraint on the map scale
+     * of the canvas will be ignored during the zoom operation.
      */
-    void zoomScale( double scale );
+    void zoomScale( double scale, bool ignoreScaleLock = false );
 
     /**
      * Zoom with the factor supplied. Factor > 1 zooms out, interval (0,1) zooms in
-     * If point is given, re-center on it
+     * If point is given, re-center on it.
+     *
+     * If \a ignoreScaleLock is set to TRUE, then any existing constraint on the map scale
+     * of the canvas will be ignored during the zoom operation.
      */
-    void zoomByFactor( double scaleFactor, const QgsPointXY *center = nullptr );
+    void zoomByFactor( double scaleFactor, const QgsPointXY *center = nullptr, bool ignoreScaleLock = false );
 
     //! Zooms in/out with a given center
     void zoomWithCenter( int x, int y, bool zoomIn );
@@ -467,7 +495,7 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! used to determine if anti-aliasing is enabled or not
     void enableAntiAliasing( bool flag );
 
-    //! TRUE if antialising is enabled
+    //! TRUE if antialiasing is enabled
     bool antiAliasingEnabled() const { return mSettings.testFlag( QgsMapSettings::Antialiasing ); }
 
     //! sets map tile rendering flag
@@ -478,6 +506,16 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Ends pan action and redraws the canvas.
     void panActionEnd( QPoint releasePoint );
 
+#ifndef SIP_RUN
+
+    /**
+     * Starts a pan action.
+     * \note Not available in Python bindings
+     * \since QGIS 3.12
+     */
+    void panActionStart( QPoint releasePoint );
+#endif
+
     //! Called when mouse is moving and pan is activated
     void panAction( QMouseEvent *event );
 
@@ -639,17 +677,39 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     void setPreviewJobsEnabled( bool enabled );
 
+    /**
+     * Sets a list of custom drop \a handlers to use when drop events occur on the canvas.
+     * \note Not available in Python bindings
+     * \since QGIS 3.10
+     */
+    void setCustomDropHandlers( const QVector<QPointer<QgsCustomDropHandler >> &handlers ) SIP_SKIP;
+
   public slots:
 
     //! Repaints the canvas map
     void refresh();
 
     /**
-     * Reload all layers, clear the cache and refresh the canvas
+     * Reload all layers (including refreshing layer properties from their data sources),
+     * clears the cache and refreshes the canvas.
+     *
+     * \note Consider using the less expensive redrawAllLayers() method if a layer reload
+     * from the data provider is not required.
+     *
      * \since QGIS 2.9
      */
     void refreshAllLayers();
 
+    /**
+     * Clears all cached images and redraws all layers.
+     *
+     * \note Unlike refreshAllLayers(), this does NOT reload layers themselves, and accordingly
+     * is more "lightweight". Use this method when only an update of the layer's renderers is required.
+     *
+     * \since QGIS 3.10
+     */
+    void redrawAllLayers();
+
     //! Receives signal about selection change, and pass it on with layer info
     void selectionChangedSlot();
 
@@ -707,6 +767,20 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     void zoomToSelected( QgsVectorLayer *layer = nullptr );
 
+    /**
+     * Set a list of resolutions (map units per pixel) to which to "snap to" when zooming the map
+     * \param resolutions A list of resolutions
+     * \since QGIS 3.12
+     */
+    void setZoomResolutions( const QList<double> &resolutions ) { mZoomResolutions = resolutions; }
+
+    /**
+     * \returns List of resolutions to which to "snap to" when zooming the map
+     * \see setZoomResolutions()
+     * \since QGIS 3.12
+     */
+    const QList<double> &zoomResolutions() const { return mZoomResolutions; }
+
   private slots:
     //! called when current maptool is destroyed
     void mapToolDestroyed();
@@ -837,44 +911,53 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! emit a message (usually to be displayed in a message bar)
     void messageEmitted( const QString &title, const QString &message, Qgis::MessageLevel = Qgis::Info );
 
+    /**
+     * Emitted whenever an error is encountered during a map render operation.
+     *
+     * The \a layer argument indicates the associated map layer, if available.
+     *
+     * \since QGIS 3.10.0
+     */
+    void renderErrorOccurred( const QString &error, QgsMapLayer *layer );
+
+    /**
+     * Emitted whenever the distance or bearing of an in-progress panning
+     * operation is changed.
+     *
+     * This signal will be emitted during a pan operation as the user moves the map,
+     * giving the total distance and bearing between the map position at the
+     * start of the pan and the current pan position.
+     *
+     * \since QGIS 3.12
+     */
+    void panDistanceBearingChanged( double distance, QgsUnitTypes::DistanceUnit unit, double bearing );
+
+    /**
+     * Emitted whenever a tap and hold \a gesture occurs at the specified map point.
+     * \since QGIS 3.12
+     */
+    void tapAndHoldGestureOccurred( const QgsPointXY &mapPoint, QTapAndHoldGesture *gesture );
+
   protected:
 
-    //! Overridden standard event to be gestures aware
     bool event( QEvent *e ) override;
-
-    //! Overridden key press event
     void keyPressEvent( QKeyEvent *e ) override;
-
-    //! Overridden key release event
     void keyReleaseEvent( QKeyEvent *e ) override;
-
-    //! Overridden mouse double-click event
     void mouseDoubleClickEvent( QMouseEvent *e ) override;
-
-    //! Overridden mouse move event
     void mouseMoveEvent( QMouseEvent *e ) override;
-
-    //! Overridden mouse press event
     void mousePressEvent( QMouseEvent *e ) override;
-
-    //! Overridden mouse release event
     void mouseReleaseEvent( QMouseEvent *e ) override;
-
-    //! Overridden mouse wheel event
     void wheelEvent( QWheelEvent *e ) override;
-
-    //! Overridden resize event
     void resizeEvent( QResizeEvent *e ) override;
-
-    //! Overridden paint event
     void paintEvent( QPaintEvent *e ) override;
-
-    //! Overridden drag enter event
     void dragEnterEvent( QDragEnterEvent *e ) override;
 
     //! called when panning is in action, reset indicates end of panning
     void moveCanvasContents( bool reset = false );
 
+    void dropEvent( QDropEvent *event ) override;
+
+
     /// implementation struct
     class CanvasProperties;
 
@@ -1005,6 +1088,11 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
 
     QHash< QString, int > mLastLayerRenderTime;
 
+    QVector<QPointer<QgsCustomDropHandler >> mDropHandlers;
+
+    QgsDistanceArea mDa;
+    QList<double> mZoomResolutions;
+
     /**
      * Returns the last cursor position on the canvas in geographical coordinates
      * \since QGIS 3.4
@@ -1046,6 +1134,15 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     void stopPreviewJobs();
     void schedulePreviewJob( int number );
 
+    /**
+     * Returns TRUE if a pan operation is in progress
+     */
+    bool panOperationInProgress();
+
+    int nextZoomLevel( const QList<double> &resolutions, bool zoomIn = true ) const;
+    double zoomInFactor() const;
+    double zoomOutFactor() const;
+
     friend class TestQgsMapCanvas;
 
 }; // class QgsMapCanvas
