@@ -19,8 +19,11 @@
 #define QGSPREVIEWEFFECT_H
 
 #include <QGraphicsEffect>
+#include "qgis_sip.h"
+#include "qgis_gui.h"
 
-/** \ingroup gui
+/**
+ * \ingroup gui
  * A graphics effect which can be applied to a widget to simulate various printing and
  * color blindness modes.
  */
@@ -38,24 +41,26 @@ class GUI_EXPORT QgsPreviewEffect: public QGraphicsEffect
       PreviewDeuteranope
     };
 
-    QgsPreviewEffect( QObject* parent );
-    ~QgsPreviewEffect();
+    QgsPreviewEffect( QObject *parent SIP_TRANSFERTHIS );
 
-    /** Sets the mode for the preview effect, which controls how the effect modifies a widgets appearance.
-     * @param mode PreviewMode to use to draw the widget
-     * @note added in 2.3
-     * @see mode
+    /**
+     * Sets the mode for the preview effect, which controls how the effect modifies a widgets appearance.
+     * \param mode PreviewMode to use to draw the widget
+     * \see mode
+     * \since QGIS 2.3
      */
     void setMode( PreviewMode mode );
-    /** Returns the mode used for the preview effect.
-     * @returns PreviewMode currently used by the effect
-     * @note added in 2.3
-     * @see setMode
+
+    /**
+     * Returns the mode used for the preview effect.
+     * \returns PreviewMode currently used by the effect
+     * \see setMode
+     * \since QGIS 2.3
      */
     PreviewMode mode() const { return mMode; }
 
   protected:
-    virtual void draw( QPainter *painter ) override;
+    void draw( QPainter *painter ) override;
 
   private:
 
