@@ -17,12 +17,14 @@
 #define QGSRELATIONWIDGETWRAPPER_H
 
 #include "qgswidgetwrapper.h"
+#include "qgis_sip.h"
+#include "qgis_gui.h"
 
 class QgsRelationEditorWidget;
 
-/** \ingroup gui
+/**
+ * \ingroup gui
  * \class QgsRelationWidgetWrapper
- * \note not available in Python bindings
  */
 
 class GUI_EXPORT QgsRelationWidgetWrapper : public QgsWidgetWrapper
@@ -30,67 +32,81 @@ class GUI_EXPORT QgsRelationWidgetWrapper : public QgsWidgetWrapper
     Q_OBJECT
 
   public:
-    explicit QgsRelationWidgetWrapper( QgsVectorLayer* vl, const QgsRelation& relation, QWidget* editor = nullptr, QWidget* parent = nullptr );
+
+    //! Constructor for QgsRelationWidgetWrapper
+    explicit QgsRelationWidgetWrapper( QgsVectorLayer *vl, const QgsRelation &relation, QWidget *editor = nullptr, QWidget *parent SIP_TRANSFERTHIS = nullptr );
 
     /**
-     * Defines if a title lable should be shown for this widget.
+     * Defines if a title label should be shown for this widget.
      * Only has an effect after widget() has been called at least once.
      *
-     * @note Added in QGIS 2.18
+     * \since QGIS 2.18
      */
     bool showLabel() const;
 
     /**
-     * Defines if a title lable should be shown for this widget.
+     * Defines if a title label should be shown for this widget.
      * Only has an effect after widget() has been called at least once.
      *
-     * @note Added in QGIS 2.18
+     * \since QGIS 2.18
      */
     void setShowLabel( bool showLabel );
 
     /**
      * Determines if the "link feature" button should be shown
      *
-     * @note Added in QGIS 2.18
+     * \since QGIS 2.18
      */
     bool showLinkButton() const;
+
     /**
      * Determines if the "link feature" button should be shown
      *
-     * @note Added in QGIS 2.18
+     * \since QGIS 2.18
      */
     void setShowLinkButton( bool showLinkButton );
 
     /**
      * Determines if the "unlink feature" button should be shown
      *
-     * @note Added in QGIS 2.18
+     * \since QGIS 2.18
      */
     bool showUnlinkButton() const;
+
     /**
      * Determines if the "unlink feature" button should be shown
      *
-     * @note Added in QGIS 2.18
+     * \since QGIS 2.18
      */
     void setShowUnlinkButton( bool showUnlinkButton );
 
+    /**
+     * The relation for which this wrapper is created.
+     *
+     * \since QGIS 3.0
+     */
+    QgsRelation relation() const;
+
   protected:
-    QWidget* createWidget( QWidget* parent ) override;
-    void initWidget( QWidget* editor ) override;
+    QWidget *createWidget( QWidget *parent ) override;
+    void initWidget( QWidget *editor ) override;
     bool valid() const override;
 
   public slots:
-    void setFeature( const QgsFeature& feature ) override;
+    void setFeature( const QgsFeature &feature ) override;
 
-    /** Sets the visibility of the wrapper's widget.
-     * @param visible set to true to show widget, false to hide widget
-     * @note added in QGIS 2.16
+    /**
+     * Sets the visibility of the wrapper's widget.
+     * \param visible set to TRUE to show widget, FALSE to hide widget
+     * \since QGIS 2.16
      */
     void setVisible( bool visible );
 
   private:
+    void aboutToSave() override;
     QgsRelation mRelation;
-    QgsRelationEditorWidget* mWidget;
+    QgsRelation mNmRelation;
+    QgsRelationEditorWidget *mWidget = nullptr;
 };
 
 #endif // QGSRELATIONWIDGETWRAPPER_H
