@@ -95,12 +95,13 @@ class GUI_EXPORT QgsSymbolSelectorWidget: public QgsPanelWidget, private Ui::Qgs
     // TODO QGIS 4.0 - transfer ownership of symbol to widget!
 
     /**
-       * Symbol selector widget that can be used to select and build a symbol
-       * \param symbol The symbol to load into the widget as a start point.
-       * \param style The style used by the widget.
-       * \param vl The vector layer for the symbol.
-       * \param parent
-       */
+     * Symbol selector widget that can be used to select and build a symbol
+     * \param symbol The symbol to load into the widget as a start point.
+     * \param style The style used by the widget.
+     * \param vl The vector layer for the symbol.
+     * \param parent
+     * \note The ownership of the symbol is not transferred and must exist for the lifetime of the widget.
+     */
     QgsSymbolSelectorWidget( QgsSymbol *symbol, QgsStyle *style, QgsVectorLayer *vl, QWidget *parent SIP_TRANSFERTHIS = nullptr );
 
     //! Returns menu for "advanced" button - create it if doesn't exist and show the advanced button
@@ -127,59 +128,16 @@ class GUI_EXPORT QgsSymbolSelectorWidget: public QgsPanelWidget, private Ui::Qgs
      */
     QgsSymbol *symbol() { return mSymbol; }
 
+    // TODO QGIS 4.0 - transfer ownership of symbol to widget!
+
     /**
-     * Load the given symbol into the widget.
+     * Loads the given symbol into the widget.
      * \param symbol The symbol to load.
      * \param parent The parent symbol layer item. If the parent parameter is null, the whole symbol and model will be reset.
-     * \note not available in Python bindings
+     * \note The ownership of the symbol is not transferred and must exist for the lifetime of the widget.
      */
     void loadSymbol( QgsSymbol *symbol, SymbolLayerItem *parent = nullptr ) SIP_SKIP;
 
-  protected:
-
-    /**
-     * Reload the current symbol in the view.
-     */
-    void loadSymbol();
-
-    /**
-     * Update the state of the UI based on the currently set symbol layer.
-     */
-    void updateUi();
-
-    /**
-     * Update the lock button states based on the current symbol layer.
-     */
-    void updateLockButton();
-
-    //! \note not available in Python bindings
-    SymbolLayerItem *currentLayerItem() SIP_SKIP;
-
-    /**
-     * The current symbol layer that is active in the interface.
-     * \returns The active symbol layer.
-     */
-    QgsSymbolLayer *currentLayer();
-
-    /**
-     * Move the current active layer by a set offset in the list.
-     * \param offset The offset to move the layer by
-     */
-    void moveLayerByOffset( int offset );
-
-    /**
-     * Set the properties widget for the active symbol layer.
-     * \param widget The widget to set to configure the active symbol layer.
-     */
-    void setWidget( QWidget *widget );
-
-  signals:
-
-    /**
-     * Emiited when a symbol is modified in the widget.
-     */
-    void symbolModified();
-
   public slots:
 
     /**
@@ -238,8 +196,50 @@ class GUI_EXPORT QgsSymbolSelectorWidget: public QgsPanelWidget, private Ui::Qgs
      */
     void changeLayer( QgsSymbolLayer *layer );
 
+  signals:
+
+    /**
+     * Emitted when a symbol is modified in the widget.
+     */
+    void symbolModified();
+
+  private:
+
+    /**
+     * Reload the current symbol in the view.
+     */
+    void reloadSymbol();
+
+    /**
+     * Update the state of the UI based on the currently set symbol layer.
+     */
+    void updateUi();
+
+    /**
+     * Update the lock button states based on the current symbol layer.
+     */
+    void updateLockButton();
+
+    SymbolLayerItem *currentLayerItem();
+
+    /**
+     * The current symbol layer that is active in the interface.
+     * \returns The active symbol layer.
+     */
+    QgsSymbolLayer *currentLayer();
+
+    /**
+     * Move the current active layer by a set offset in the list.
+     * \param offset The offset to move the layer by
+     */
+    void moveLayerByOffset( int offset );
+
+    /**
+     * Set the properties widget for the active symbol layer.
+     * \param widget The widget to set to configure the active symbol layer.
+     */
+    void setWidget( QWidget *widget );
 
-  protected: // data
     QgsStyle *mStyle = nullptr;
     QgsSymbol *mSymbol = nullptr;
     QMenu *mAdvancedMenu = nullptr;
@@ -248,7 +248,6 @@ class GUI_EXPORT QgsSymbolSelectorWidget: public QgsPanelWidget, private Ui::Qgs
     QStandardItemModel *model = nullptr;
     QWidget *mPresentWidget = nullptr;
 
-  private:
     std::unique_ptr<DataDefinedRestorer> mDataDefineRestorer;
     QgsSymbolWidgetContext mContext;
     QgsFeature mPreviewFeature;
@@ -302,31 +301,21 @@ class GUI_EXPORT QgsSymbolSelectorDialog : public QDialog
      */
     QgsSymbol *symbol();
 
-  protected:
-    // Reimplements dialog keyPress event so we can ignore it
-    void keyPressEvent( QKeyEvent *e ) override;
+    /**
+     * Loads the given symbol into the widget.
+     * \param symbol The symbol to load.
+     * \param parent The parent symbol layer item. If the parent parameter is null, the whole symbol and model will be reset.
+     */
+    void loadSymbol( QgsSymbol *symbol, SymbolLayerItem *parent = nullptr ) SIP_SKIP;
 
-    void loadSymbol();
-
-    //! \note not available in Python bindings
-    void loadSymbol( QgsSymbol *symbol, SymbolLayerItem *parent ) SIP_SKIP;
-
-    void updateUi();
-
-    void updateLockButton();
-
-    //! \note not available in Python bindings
-    SymbolLayerItem *currentLayerItem() SIP_SKIP;
-    QgsSymbolLayer *currentLayer();
-
-    void moveLayerByOffset( int offset );
-
-    void setWidget( QWidget *widget );
-
-  signals:
-    void symbolModified();
+    /**
+     * Returns a reference to the dialog's button box.
+     * \since QGIS 3.10
+     */
+    QDialogButtonBox *buttonBox() const;
 
   public slots:
+
     void moveLayerDown();
     void moveLayerUp();
 
@@ -355,13 +344,39 @@ class GUI_EXPORT QgsSymbolSelectorDialog : public QDialog
      */
     void changeLayer( QgsSymbolLayer *layer );
 
+  protected:
+
+    // Reimplements dialog keyPress event so we can ignore it
+    void keyPressEvent( QKeyEvent *e ) override;
+
+  private slots:
+
+    void showHelp();
+
+  signals:
+
+    void symbolModified();
+
   private:
+
+    void reloadSymbol();
+
+    void updateUi();
+
+    void updateLockButton();
+
+    SymbolLayerItem *currentLayerItem();
+
+    QgsSymbolLayer *currentLayer();
+
+    void moveLayerByOffset( int offset );
+
+    void setWidget( QWidget *widget );
+
     QgsSymbolSelectorWidget *mSelectorWidget = nullptr;
     QDialogButtonBox *mButtonBox = nullptr;
     QgsSymbolWidgetContext mContext;
 
-  private slots:
-    void showHelp();
 };
 
 #endif
