@@ -3,13 +3,13 @@
 #
 # CSW Client
 # ---------------------------------------------------------
-# QGIS Catalogue Service client.
+# QGIS Catalog Service client.
 #
 # Copyright (C) 2010 NextGIS (http://nextgis.org),
 #                    Alexander Bruy (alexander.bruy@gmail.com),
 #                    Maxim Dubinin (sim@gis-lab.info)
 #
-# Copyright (C) 2014 Tom Kralidis (tomkralidis@gmail.com)
+# Copyright (C) 2017 Tom Kralidis (tomkralidis@gmail.com)
 #
 # This source is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -29,18 +29,26 @@
 
 import json
 import os.path
-from urllib2 import build_opener, install_opener, ProxyHandler
+import warnings
+from urllib.request import build_opener, install_opener, ProxyHandler
 
-from qgis.PyQt.QtCore import QSettings, Qt
-from qgis.PyQt.QtWidgets import QApplication, QDialog, QDialogButtonBox, QMessageBox, QTreeWidgetItem, QWidget
+from qgis.PyQt.QtCore import Qt
+from qgis.PyQt.QtWidgets import (QApplication, QDialog, QComboBox,
+                                 QDialogButtonBox, QMessageBox,
+                                 QTreeWidgetItem, QWidget)
 from qgis.PyQt.QtGui import QColor, QCursor
 
 from qgis.core import (QgsApplication, QgsCoordinateReferenceSystem,
-                       QgsCoordinateTransform, QgsGeometry, QgsPoint,
-                       QgsProviderRegistry)
-from qgis.gui import QgsRubberBand
+                       QgsCoordinateTransform, QgsGeometry, QgsPointXY,
+                       QgsProviderRegistry, QgsSettings, QgsProject)
+from qgis.gui import QgsRubberBand, QgsGui
+from qgis.utils import OverrideCursor
+
+with warnings.catch_warnings():
+    warnings.filterwarnings("ignore", category=ResourceWarning)
+    warnings.filterwarnings("ignore", category=ImportWarning)
+    from owslib.csw import CatalogueServiceWeb # spellok
 
-from owslib.csw import CatalogueServiceWeb
 from owslib.fes import BBox, PropertyIsLike
 from owslib.ows import ExceptionReport
 
@@ -49,10 +57,10 @@ from MetaSearch.dialogs.manageconnectionsdialog import ManageConnectionsDialog
 from MetaSearch.dialogs.newconnectiondialog import NewConnectionDialog
 from MetaSearch.dialogs.recorddialog import RecordDialog
 from MetaSearch.dialogs.xmldialog import XMLDialog
-from MetaSearch.util import (get_connections_from_file, get_ui_class,
-                             get_help_url, highlight_xml, normalize_text,
-                             open_url, render_template, serialize_string,
-                             StaticContext)
+from MetaSearch.util import (clean_ows_url, get_connections_from_file,
+                             get_ui_class, get_help_url, highlight_xml,
+                             normalize_text, open_url, render_template,
+                             serialize_string, StaticContext)
 
 BASE_CLASS = get_ui_class('maindialog.ui')
 
@@ -69,13 +77,17 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         self.iface = iface
         self.map = iface.mapCanvas()
-        self.settings = QSettings()
+        self.settings = QgsSettings()
         self.catalog = None
         self.catalog_url = None
+        self.catalog_username = None
+        self.catalog_password = None
         self.context = StaticContext()
 
-        version = self.context.metadata.get('general', 'version')
-        self.setWindowTitle('MetaSearch %s' % version)
+        self.leKeywords.setShowSearchIcon(True)
+        self.leKeywords.setPlaceholderText(self.tr('Search keywords'))
+
+        self.setWindowTitle(self.tr('MetaSearch'))
 
         self.rubber_band = QgsRubberBand(self.map, True)  # True = a polygon
         self.rubber_band.setColor(QColor(255, 0, 0, 75))
@@ -121,16 +133,14 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.btnNext.clicked.connect(self.navigate)
         self.btnLast.clicked.connect(self.navigate)
 
-        self.btnAddToWms.clicked.connect(self.add_to_ows)
-        self.btnAddToWfs.clicked.connect(self.add_to_ows)
-        self.btnAddToWcs.clicked.connect(self.add_to_ows)
+        self.mActionAddWms.triggered.connect(self.add_to_ows)
+        self.mActionAddWfs.triggered.connect(self.add_to_ows)
+        self.mActionAddWcs.triggered.connect(self.add_to_ows)
+        self.mActionAddAms.triggered.connect(self.add_to_ows)
+        self.mActionAddAfs.triggered.connect(self.add_to_ows)
+        self.mActionAddGisFile.triggered.connect(self.add_gis_file)
         self.btnShowXml.clicked.connect(self.show_xml)
 
-        # settings
-        self.radioTitleAsk.clicked.connect(self.set_ows_save_title_ask)
-        self.radioTitleNoAsk.clicked.connect(self.set_ows_save_title_no_ask)
-        self.radioTempName.clicked.connect(self.set_ows_save_temp_name)
-
         self.manageGui()
 
     def manageGui(self):
@@ -140,25 +150,17 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.populate_connection_list()
         self.btnCapabilities.setEnabled(False)
         self.spnRecords.setValue(
-            self.settings.value('/MetaSearch/returnRecords', 10, int))
+            int(self.settings.value('/MetaSearch/returnRecords', 10)))
 
         key = '/MetaSearch/%s' % self.cmbConnectionsSearch.currentText()
         self.catalog_url = self.settings.value('%s/url' % key)
+        self.catalog_username = self.settings.value('%s/username' % key)
+        self.catalog_password = self.settings.value('%s/password' % key)
 
         self.set_bbox_global()
 
         self.reset_buttons()
 
-        # get preferred connection save strategy from settings and set it
-        save_strat = self.settings.value('/MetaSearch/ows_save_strategy',
-                                         'title_ask')
-        if save_strat == 'temp_name':
-            self.radioTempName.setChecked(True)
-        elif save_strat == 'title_no_ask':
-            self.radioTitleNoAsk.setChecked(True)
-        else:
-            self.radioTitleAsk.setChecked(True)
-
         # install proxy handler if specified in QGIS settings
         self.install_proxy()
 
@@ -221,7 +223,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # lot of items)
         if not exists and conn_count > 0:
             # If to_select is null, then the selected connection wasn't found
-            # by QSettings, which probably means that this is the first time
+            # by QgsSettings, which probably means that this is the first time
             # the user has used CSWClient, so default to the first in the list
             # of connetions. Otherwise default to the last.
             if not to_select:
@@ -247,6 +249,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         if caller == 'cmbConnectionsSearch':  # bind to service in search tab
             self.catalog_url = self.settings.value('%s/url' % key)
+            self.catalog_username = self.settings.value('%s/username' % key)
+            self.catalog_password = self.settings.value('%s/password' % key)
 
         if caller == 'cmbConnectionsServices':  # clear server metadata
             self.textMetadata.clear()
@@ -259,13 +263,13 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         current_text = self.cmbConnectionsServices.currentText()
         key = '/MetaSearch/%s' % current_text
         self.catalog_url = self.settings.value('%s/url' % key)
+        self.catalog_username = self.settings.value('%s/username' % key)
+        self.catalog_password = self.settings.value('%s/password' % key)
 
         # connect to the server
         if not self._get_csw():
             return
 
-        QApplication.restoreOverrideCursor()
-
         if self.catalog:  # display service metadata
             self.btnCapabilities.setEnabled(True)
             metadata = render_template('en', self.context,
@@ -280,7 +284,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         """add new service"""
 
         conn_new = NewConnectionDialog()
-        conn_new.setWindowTitle(self.tr('New Catalogue service'))
+        conn_new.setWindowTitle(self.tr('New Catalog Service'))
         if conn_new.exec_() == QDialog.Accepted:  # add to service list
             self.populate_connection_list()
         self.textMetadata.clear()
@@ -293,9 +297,12 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         url = self.settings.value('/MetaSearch/%s/url' % current_text)
 
         conn_edit = NewConnectionDialog(current_text)
-        conn_edit.setWindowTitle(self.tr('Edit Catalogue service'))
+        conn_edit.setWindowTitle(self.tr('Edit Catalog Service'))
         conn_edit.leName.setText(current_text)
         conn_edit.leURL.setText(url)
+        conn_edit.leUsername.setText(self.settings.value('/MetaSearch/%s/username' % current_text))
+        conn_edit.lePassword.setText(self.settings.value('/MetaSearch/%s/password' % current_text))
+
         if conn_edit.exec_() == QDialog.Accepted:  # update service list
             self.populate_connection_list()
 
@@ -306,11 +313,11 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         key = '/MetaSearch/%s' % current_text
 
-        msg = self.tr('Remove service %s?') % current_text
+        msg = self.tr('Remove service {0}?').format(current_text)
 
-        result = QMessageBox.information(self, self.tr('Confirm delete'), msg,
-                                         QMessageBox.Ok | QMessageBox.Cancel)
-        if result == QMessageBox.Ok:  # remove service from list
+        result = QMessageBox.question(self, self.tr('Delete Service'), msg,
+                                      QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
+        if result == QMessageBox.Yes:  # remove service from list
             self.settings.remove(key)
             index_to_delete = self.cmbConnectionsServices.currentIndex()
             self.cmbConnectionsServices.removeItem(index_to_delete)
@@ -340,7 +347,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             name = server.attrib.get('name')
             # check for duplicates
             if name in keys:
-                msg = self.tr('%s exists.  Overwrite?') % name
+                msg = self.tr('{0} exists.  Overwrite?').format(name)
                 res = QMessageBox.warning(self,
                                           self.tr('Loading connections'), msg,
                                           QMessageBox.Yes | QMessageBox.No)
@@ -375,19 +382,22 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
     def set_bbox_from_map(self):
         """set bounding box from map extent"""
 
-        crs = self.map.mapRenderer().destinationCrs()
-        crsid = int(crs.authid().split(':')[1])
+        crs = self.map.mapSettings().destinationCrs()
+        try:
+            crsid = int(crs.authid().split(':')[1])
+        except IndexError:  # no projection
+            crsid = 4326
 
         extent = self.map.extent()
 
         if crsid != 4326:  # reproject to EPSG:4326
             src = QgsCoordinateReferenceSystem(crsid)
             dest = QgsCoordinateReferenceSystem(4326)
-            xform = QgsCoordinateTransform(src, dest)
-            minxy = xform.transform(QgsPoint(extent.xMinimum(),
-                                             extent.yMinimum()))
-            maxxy = xform.transform(QgsPoint(extent.xMaximum(),
-                                             extent.yMaximum()))
+            xform = QgsCoordinateTransform(src, dest, QgsProject.instance())
+            minxy = xform.transform(QgsPointXY(extent.xMinimum(),
+                                               extent.yMinimum()))
+            maxxy = xform.transform(QgsPointXY(extent.xMaximum(),
+                                               extent.yMaximum()))
             minx, miny = minxy
             maxx, maxy = maxxy
         else:  # 4326
@@ -396,10 +406,10 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             maxx = extent.xMaximum()
             maxy = extent.yMaximum()
 
-        self.leNorth.setText(unicode(maxy)[0:9])
-        self.leSouth.setText(unicode(miny)[0:9])
-        self.leWest.setText(unicode(minx)[0:9])
-        self.leEast.setText(unicode(maxx)[0:9])
+        self.leNorth.setText(str(maxy)[0:9])
+        self.leSouth.setText(str(miny)[0:9])
+        self.leWest.setText(str(minx)[0:9])
+        self.leEast.setText(str(maxx)[0:9])
 
     def set_bbox_global(self):
         """set global bounding box"""
@@ -424,10 +434,12 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.settings.setValue('/MetaSearch/returnRecords',
                                self.spnRecords.cleanText())
 
-        # set current catalogue
+        # set current catalog
         current_text = self.cmbConnectionsSearch.currentText()
         key = '/MetaSearch/%s' % current_text
         self.catalog_url = self.settings.value('%s/url' % key)
+        self.catalog_username = self.settings.value('%s/username' % key)
+        self.catalog_password = self.settings.value('%s/password' % key)
 
         # start position and number of records to return
         self.startfrom = 0
@@ -437,6 +449,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.timeout = self.spnTimeout.value()
 
         # bbox
+        # CRS is WGS84 with axis order longitude, latitude
+        # defined by 'urn:ogc:def:crs:OGC:1.3:CRS84'
         minx = self.leWest.text()
         miny = self.leSouth.text()
         maxx = self.leEast.text()
@@ -447,7 +461,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # even for a global bbox, if a spatial filter is applied, then
         # the CSW server will skip records without a bbox
         if bbox != ['-180', '-90', '180', '90']:
-            self.constraints.append(BBox(bbox))
+            self.constraints.append(BBox(bbox,
+                                         crs='urn:ogc:def:crs:OGC:1.3:CRS84'))
 
         # keywords
         if self.leKeywords.text():
@@ -465,25 +480,22 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # TODO: allow users to select resources types
         # to find ('service', 'dataset', etc.)
         try:
-            self.catalog.getrecords2(constraints=self.constraints,
-                                     maxrecords=self.maxrecords, esn='full')
+            with OverrideCursor(Qt.WaitCursor):
+                self.catalog.getrecords2(constraints=self.constraints,
+                                         maxrecords=self.maxrecords, esn='full')
         except ExceptionReport as err:
-            QApplication.restoreOverrideCursor()
             QMessageBox.warning(self, self.tr('Search error'),
-                                self.tr('Search error: %s') % err)
+                                self.tr('Search error: {0}').format(err))
             return
         except Exception as err:
-            QApplication.restoreOverrideCursor()
             QMessageBox.warning(self, self.tr('Connection error'),
-                                self.tr('Connection error: %s') % err)
+                                self.tr('Connection error: {0}').format(err))
             return
 
         if self.catalog.results['matches'] == 0:
-            QApplication.restoreOverrideCursor()
             self.lblResults.setText(self.tr('0 results'))
             return
 
-        QApplication.restoreOverrideCursor()
         self.display_results()
 
     def display_results(self):
@@ -493,9 +505,9 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         position = self.catalog.results['returned'] + self.startfrom
 
-        msg = self.tr('Showing %d - %d of %n result(s)', 'number of results',
-                      self.catalog.results['matches']) % (self.startfrom + 1,
-                                                          position)
+        msg = self.tr('Showing {0} - {1} of %n result(s)', 'number of results',
+                      self.catalog.results['matches']).format(self.startfrom + 1,
+                                                              position)
 
         self.lblResults.setText(msg)
 
@@ -551,17 +563,17 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             points = bbox_to_polygon(record.bbox)
             if points is not None:
                 src = QgsCoordinateReferenceSystem(4326)
-                dst = self.map.mapRenderer().destinationCrs()
-                geom = QgsGeometry.fromPolygon(points)
+                dst = self.map.mapSettings().destinationCrs()
+                geom = QgsGeometry.fromWkt(points)
                 if src.postgisSrid() != dst.postgisSrid():
-                    ctr = QgsCoordinateTransform(src, dst)
+                    ctr = QgsCoordinateTransform(src, dst, QgsProject.instance())
                     try:
                         geom.transform(ctr)
                     except Exception as err:
                         QMessageBox.warning(
                             self,
                             self.tr('Coordinate Transformation Error'),
-                            unicode(err))
+                            str(err))
                 self.rubber_band.setToGeometry(geom, None)
 
         # figure out if the data is interactive and can be operated on
@@ -585,24 +597,38 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             if link_type is not None:
                 link_type = link_type.upper()
 
-            wmswmst_link_types = map(str.upper, link_types.WMSWMST_LINK_TYPES)
-            wfs_link_types = map(str.upper, link_types.WFS_LINK_TYPES)
-            wcs_link_types = map(str.upper, link_types.WCS_LINK_TYPES)
+            wmswmst_link_types = list(map(str.upper, link_types.WMSWMST_LINK_TYPES))
+            wfs_link_types = list(map(str.upper, link_types.WFS_LINK_TYPES))
+            wcs_link_types = list(map(str.upper, link_types.WCS_LINK_TYPES))
+            ams_link_types = list(map(str.upper, link_types.AMS_LINK_TYPES))
+            afs_link_types = list(map(str.upper, link_types.AFS_LINK_TYPES))
+            gis_file_link_types = list(map(str.upper, link_types.GIS_FILE_LINK_TYPES))
 
             # if the link type exists, and it is one of the acceptable
             # interactive link types, then set
             if all([link_type is not None,
                     link_type in wmswmst_link_types + wfs_link_types +
-                    wcs_link_types]):
+                    wcs_link_types + ams_link_types + afs_link_types + gis_file_link_types]):
                 if link_type in wmswmst_link_types:
                     services['wms'] = link['url']
-                    self.btnAddToWms.setEnabled(True)
+                    self.mActionAddWms.setEnabled(True)
                 if link_type in wfs_link_types:
                     services['wfs'] = link['url']
-                    self.btnAddToWfs.setEnabled(True)
+                    self.mActionAddWfs.setEnabled(True)
                 if link_type in wcs_link_types:
                     services['wcs'] = link['url']
-                    self.btnAddToWcs.setEnabled(True)
+                    self.mActionAddWcs.setEnabled(True)
+                if link_type in ams_link_types:
+                    services['ams'] = link['url']
+                    self.mActionAddAms.setEnabled(True)
+                if link_type in afs_link_types:
+                    services['afs'] = link['url']
+                    self.mActionAddAfs.setEnabled(True)
+                if link_type in gis_file_link_types:
+                    services['gis_file'] = link['url']
+                    services['title'] = record.title
+                    self.mActionAddGisFile.setEnabled(True)
+                self.tbAddData.setEnabled(True)
 
             set_item_data(item, 'link', json.dumps(services))
 
@@ -641,25 +667,20 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
                 else:
                     return
 
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-
         try:
-            self.catalog.getrecords2(constraints=self.constraints,
-                                     maxrecords=self.maxrecords,
-                                     startposition=self.startfrom, esn='full')
+            with OverrideCursor(Qt.WaitCursor):
+                self.catalog.getrecords2(constraints=self.constraints,
+                                         maxrecords=self.maxrecords,
+                                         startposition=self.startfrom, esn='full')
         except ExceptionReport as err:
-            QApplication.restoreOverrideCursor()
             QMessageBox.warning(self, self.tr('Search error'),
-                                self.tr('Search error: %s') % err)
+                                self.tr('Search error: {0}').format(err))
             return
         except Exception as err:
-            QApplication.restoreOverrideCursor()
             QMessageBox.warning(self, self.tr('Connection error'),
-                                self.tr('Connection error: %s') % err)
+                                self.tr('Connection error: {0}').format(err))
             return
 
-        QApplication.restoreOverrideCursor()
-
         self.display_results()
 
     def add_to_ows(self):
@@ -676,24 +697,31 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         caller = self.sender().objectName()
 
-        # stype = human name,/Qgis/connections-%s,providername
-        if caller == 'btnAddToWms':
+        # stype = human name,/qgis/connections-%s,providername
+        if caller == 'mActionAddWms':
             stype = ['OGC:WMS/OGC:WMTS', 'wms', 'wms']
             data_url = item_data['wms']
-        elif caller == 'btnAddToWfs':
+        elif caller == 'mActionAddWfs':
             stype = ['OGC:WFS', 'wfs', 'WFS']
             data_url = item_data['wfs']
-        elif caller == 'btnAddToWcs':
+        elif caller == 'mActionAddWcs':
             stype = ['OGC:WCS', 'wcs', 'wcs']
             data_url = item_data['wcs']
-
-        QApplication.restoreOverrideCursor()
+        elif caller == 'mActionAddAms':
+            stype = ['ESRI:ArcGIS:MapServer', 'ams', 'arcgismapserver']
+            data_url = item_data['ams'].split('MapServer')[0] + 'MapServer'
+        elif caller == 'mActionAddAfs':
+            stype = ['ESRI:ArcGIS:FeatureServer', 'afs', 'arcgisfeatureserver']
+            data_url = item_data['afs'].split('FeatureServer')[0] + 'FeatureServer'
 
         sname = '%s from MetaSearch' % stype[1]
 
         # store connection
         # check if there is a connection with same name
-        self.settings.beginGroup('/Qgis/connections-%s' % stype[1])
+        if caller in ['mActionAddAms', 'mActionAddAfs']:
+            self.settings.beginGroup('/qgis/connections-%s' % stype[2])
+        else:
+            self.settings.beginGroup('/qgis/connections-%s' % stype[1])
         keys = self.settings.childGroups()
         self.settings.endGroup()
 
@@ -705,41 +733,56 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         # check for duplicates
         if sname in keys:  # duplicate found
-            if self.radioTitleAsk.isChecked():  # ask to overwrite
-                msg = self.tr('Connection %s exists. Overwrite?') % sname
-                res = QMessageBox.warning(self, self.tr('Saving server'), msg,
-                                          QMessageBox.Yes | QMessageBox.No)
-                if res != QMessageBox.Yes:  # assign new name with serial
-                    sname = serialize_string(sname)
-            elif self.radioTitleNoAsk.isChecked():  # don't ask to overwrite
-                pass
-            elif self.radioTempName.isChecked():  # use temp name
+            msg = self.tr('Connection {0} exists. Overwrite?').format(sname)
+            res = QMessageBox.warning(self, self.tr('Saving server'), msg,
+                                      QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
+            if res == QMessageBox.No:  # assign new name with serial
                 sname = serialize_string(sname)
+            elif res == QMessageBox.Cancel:
+                return
 
         # no dups detected or overwrite is allowed
-        self.settings.beginGroup('/Qgis/connections-%s' % stype[1])
-        self.settings.setValue('/%s/url' % sname, data_url)
+        if caller in ['mActionAddAms', 'mActionAddAfs']:
+            self.settings.beginGroup('/qgis/connections-%s' % stype[2])
+        else:
+            self.settings.beginGroup('/qgis/connections-%s' % stype[1])
+        self.settings.setValue('/%s/url' % sname, clean_ows_url(data_url))
         self.settings.endGroup()
 
         # open provider window
-        ows_provider = QgsProviderRegistry.instance().selectWidget(stype[2],
-                                                                   self)
-
+        ows_provider = QgsGui.sourceSelectProviderRegistry().createSelectionWidget(
+            stype[2],
+            self,
+            Qt.Widget,
+            QgsProviderRegistry.WidgetMode.Embedded)
         service_type = stype[0]
 
         # connect dialog signals to iface slots
         if service_type == 'OGC:WMS/OGC:WMTS':
             ows_provider.addRasterLayer.connect(self.iface.addRasterLayer)
             conn_cmb = ows_provider.findChild(QWidget, 'cmbConnections')
-            connect = 'on_btnConnect_clicked'
+            connect = 'btnConnect_clicked'
         elif service_type == 'OGC:WFS':
-            ows_provider.addWfsLayer.connect(self.iface.mainWindow().addWfsLayer)
+            def addVectorLayer(path, name):
+                self.iface.mainWindow().addVectorLayer(path, name, 'WFS')
+            ows_provider.addVectorLayer.connect(addVectorLayer)
             conn_cmb = ows_provider.findChild(QWidget, 'cmbConnections')
             connect = 'connectToServer'
         elif service_type == 'OGC:WCS':
             ows_provider.addRasterLayer.connect(self.iface.addRasterLayer)
             conn_cmb = ows_provider.findChild(QWidget, 'mConnectionsComboBox')
-            connect = 'on_mConnectButton_clicked'
+            connect = 'mConnectButton_clicked'
+        elif service_type == 'ESRI:ArcGIS:MapServer':
+            ows_provider.addRasterLayer.connect(self.iface.addRasterLayer)
+            conn_cmb = ows_provider.findChild(QComboBox)
+            connect = 'connectToServer'
+        elif service_type == 'ESRI:ArcGIS:FeatureServer':
+            def addAfsLayer(path, name):
+                self.iface.mainWindow().addVectorLayer(path, name, 'afs')
+            ows_provider.addVectorLayer.connect(addAfsLayer)
+            conn_cmb = ows_provider.findChild(QComboBox)
+            connect = 'connectToServer'
+
         ows_provider.setModal(False)
         ows_provider.show()
 
@@ -749,9 +792,27 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             conn_cmb.setCurrentIndex(index)
             # only for wfs
             if service_type == 'OGC:WFS':
-                ows_provider.on_cmbConnections_activated(index)
+                ows_provider.cmbConnections_activated(index)
+            elif service_type in ['ESRI:ArcGIS:MapServer', 'ESRI:ArcGIS:FeatureServer']:
+                ows_provider.cmbConnections_activated(index)
         getattr(ows_provider, connect)()
 
+    def add_gis_file(self):
+        """add GIS file from result"""
+        item = self.treeRecords.currentItem()
+
+        if not item:
+            return
+
+        item_data = json.loads(get_item_data(item, 'link'))
+        gis_file = item_data['gis_file']
+
+        title = item_data['title']
+
+        layer = self.iface.addVectorLayer(gis_file, title, "ogr")
+        if not layer:
+            self.iface.messageBar().pushWarning(None, "Layer failed to load!")
+
     def show_metadata(self):
         """show record metadata"""
 
@@ -765,24 +826,22 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         identifier = get_item_data(item, 'identifier')
 
         try:
-            QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-            cat = CatalogueServiceWeb(self.catalog_url, timeout=self.timeout)
-            cat.getrecordbyid(
-                [self.catalog.records[identifier].identifier])
+            with OverrideCursor(Qt.WaitCursor):
+                cat = CatalogueServiceWeb(self.catalog_url, timeout=self.timeout, # spellok
+                                          username=self.catalog_username,
+                                          password=self.catalog_password)
+                cat.getrecordbyid(
+                    [self.catalog.records[identifier].identifier])
         except ExceptionReport as err:
-            QApplication.restoreOverrideCursor()
             QMessageBox.warning(self, self.tr('GetRecords error'),
-                                self.tr('Error getting response: %s') % err)
+                                self.tr('Error getting response: {0}').format(err))
             return
         except KeyError as err:
             QMessageBox.warning(self,
                                 self.tr('Record parsing error'),
-                                'Unable to locate record identifier')
-            QApplication.restoreOverrideCursor()
+                                self.tr('Unable to locate record identifier'))
             return
 
-        QApplication.restoreOverrideCursor()
-
         record = cat.records[identifier]
         record.xml_url = cat.request
 
@@ -814,9 +873,13 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         """Convenience function to disable WMS/WMTS|WFS|WCS buttons"""
 
         if services:
-            self.btnAddToWms.setEnabled(False)
-            self.btnAddToWfs.setEnabled(False)
-            self.btnAddToWcs.setEnabled(False)
+            self.tbAddData.setEnabled(False)
+            self.mActionAddWms.setEnabled(False)
+            self.mActionAddWfs.setEnabled(False)
+            self.mActionAddWcs.setEnabled(False)
+            self.mActionAddAms.setEnabled(False)
+            self.mActionAddAfs.setEnabled(False)
+            self.mActionAddGisFile.setEnabled(False)
 
         if xml:
             self.btnShowXml.setEnabled(False)
@@ -839,23 +902,24 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.rubber_band.reset()
 
     def _get_csw(self):
-        """convenience function to init owslib.csw.CatalogueServiceWeb"""
+        """convenience function to init owslib.csw.CatalogueServiceWeb""" # spellok
 
         # connect to the server
-        try:
-            QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-            self.catalog = CatalogueServiceWeb(self.catalog_url,
-                                               timeout=self.timeout)
-            return True
-        except ExceptionReport as err:
-            msg = self.tr('Error connecting to service: %s') % err
-        except ValueError as err:
-            msg = self.tr('Value Error: %s') % err
-        except Exception as err:
-            msg = self.tr('Unknown Error: %s') % err
+        with OverrideCursor(Qt.WaitCursor):
+            try:
+                self.catalog = CatalogueServiceWeb(self.catalog_url, # spellok
+                                                   timeout=self.timeout,
+                                                   username=self.catalog_username,
+                                                   password=self.catalog_password)
+                return True
+            except ExceptionReport as err:
+                msg = self.tr('Error connecting to service: {0}').format(err)
+            except ValueError as err:
+                msg = self.tr('Value Error: {0}').format(err)
+            except Exception as err:
+                msg = self.tr('Unknown Error: {0}').format(err)
 
         QMessageBox.warning(self, self.tr('CSW Connection error'), msg)
-        QApplication.restoreOverrideCursor()
         return False
 
     def install_proxy(self):
@@ -918,7 +982,7 @@ def _get_field_value(field):
 
 
 def bbox_to_polygon(bbox):
-    """converts OWSLib bbox object to list of QgsPoint objects"""
+    """converts OWSLib bbox object to list of QgsPointXY objects"""
 
     if all([bbox.minx is not None,
             bbox.maxx is not None,
@@ -929,11 +993,6 @@ def bbox_to_polygon(bbox):
         maxx = float(bbox.maxx)
         maxy = float(bbox.maxy)
 
-        return [[
-            QgsPoint(minx, miny),
-            QgsPoint(minx, maxy),
-            QgsPoint(maxx, maxy),
-            QgsPoint(maxx, miny)
-        ]]
+        return 'POLYGON((%.2f %.2f, %.2f %.2f, %.2f %.2f, %.2f %.2f, %.2f %.2f))' % (minx, miny, minx, maxy, maxx, maxy, maxx, miny, minx, miny)  # noqa
     else:
         return None
