@@ -21,7 +21,6 @@ __author__ = 'Alexander Bruy'
 __date__ = 'September 2017'
 __copyright__ = '(C) 2017, Alexander Bruy'
 
-
 import os
 
 from qgis.PyQt.QtGui import QIcon
@@ -48,6 +47,7 @@ class GridLinear(GdalAlgorithm):
     RADIUS = 'RADIUS'
     NODATA = 'NODATA'
     OPTIONS = 'OPTIONS'
+    EXTRA = 'EXTRA'
     DATA_TYPE = 'DATA_TYPE'
     OUTPUT = 'OUTPUT'
 
@@ -90,6 +90,13 @@ class GridLinear(GdalAlgorithm):
                 'class': 'processing.algs.gdal.ui.RasterOptionsWidget.RasterOptionsWidgetWrapper'}})
         self.addParameter(options_param)
 
+        extra_param = QgsProcessingParameterString(self.EXTRA,
+                                                   self.tr('Additional command-line parameters'),
+                                                   defaultValue=None,
+                                                   optional=True)
+        extra_param.setFlags(extra_param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
+        self.addParameter(extra_param)
+
         dataType_param = QgsProcessingParameterEnum(self.DATA_TYPE,
                                                     self.tr('Output data type'),
                                                     self.TYPES,
@@ -148,6 +155,10 @@ class GridLinear(GdalAlgorithm):
         if options:
             arguments.extend(GdalUtils.parseCreationOptions(options))
 
+        if self.EXTRA in parameters and parameters[self.EXTRA] not in (None, ''):
+            extra = self.parameterAsString(parameters, self.EXTRA, context)
+            arguments.append(extra)
+
         arguments.append(ogrLayer)
         arguments.append(out)
 
