@@ -50,6 +50,7 @@ class Grass7Utils:
     GRASS_LOG_COMMANDS = 'GRASS7_LOG_COMMANDS'
     GRASS_LOG_CONSOLE = 'GRASS7_LOG_CONSOLE'
     GRASS_HELP_PATH = 'GRASS_HELP_PATH'
+    GRASS_USE_REXTERNAL = 'GRASS_USE_REXTERNAL'
     GRASS_USE_VEXTERNAL = 'GRASS_USE_VEXTERNAL'
 
     # TODO Review all default options formats
@@ -161,8 +162,8 @@ class Grass7Utils:
                 ]
         else:
             cmdList = [
-                "grass76", "grass74", "grass72", "grass70", "grass",
-                "grass76.sh", "grass74.sh", "grass72.sh", "grass70.sh", "grass.sh"
+                "grass78", "grass76", "grass74", "grass72", "grass70", "grass",
+                "grass78.sh", "grass76.sh", "grass74.sh", "grass72.sh", "grass70.sh", "grass.sh"
             ]
 
         # For MS-Windows there is a difference between GRASS Path and GRASS binary
@@ -226,7 +227,7 @@ class Grass7Utils:
             elif isMac():
                 # For MacOSX, we scan some well-known directories
                 # Start with QGIS bundle
-                for version in ['', '7', '76', '74', '72', '71', '70']:
+                for version in ['', '7', '78', '76', '74', '72', '71', '70']:
                     testfolder = os.path.join(str(QgsApplication.prefixPath()),
                                               'grass{}'.format(version))
                     if os.path.isdir(testfolder):
@@ -234,7 +235,7 @@ class Grass7Utils:
                         break
                     # If nothing found, try standalone GRASS installation
                     if folder is None:
-                        for version in ['6', '4', '2', '1', '0']:
+                        for version in ['8', '6', '4', '2', '1', '0']:
                             testfolder = '/Applications/GRASS-7.{}.app/Contents/MacOS'.format(version)
                             if os.path.isdir(testfolder):
                                 folder = testfolder
@@ -401,6 +402,9 @@ class Grass7Utils:
                     elif 'Segmentation fault' in line:
                         feedback.reportError(line.strip())
                         feedback.reportError('\n' + Grass7Utils.tr('GRASS command crashed :( Try a different set of input parameters and consult the GRASS algorithm manual for more information.') + '\n')
+                        if ProcessingConfig.getSetting(Grass7Utils.GRASS_USE_REXTERNAL):
+                            feedback.reportError(Grass7Utils.tr(
+                                'Suggest disabling the experimental "use r.external" option from the Processing GRASS Provider options.') + '\n')
                         if ProcessingConfig.getSetting(Grass7Utils.GRASS_USE_VEXTERNAL):
                             feedback.reportError(Grass7Utils.tr(
                                 'Suggest disabling the experimental "use v.external" option from the Processing GRASS Provider options.') + '\n')
@@ -561,7 +565,7 @@ class Grass7Utils:
             return 'https://grass.osgeo.org/grass{}/manuals/'.format(version)
         else:
             # GRASS not available!
-            return 'https://grass.osgeo.org/grass76/manuals/'
+            return 'https://grass.osgeo.org/grass78/manuals/'
 
     @staticmethod
     def getSupportedOutputRasterExtensions():
