@@ -20,10 +20,6 @@
 __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
-# This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '$Format:%H$'
-
-# -*- coding: utf-8 -*-
 
 #############################################################################
 #
@@ -59,7 +55,8 @@ __revision__ = '$Format:%H$'
 
 
 def usage():
-    print """
+    # fix_print_with_import
+    print("""
 voronoi - compute Voronoi diagram or Delaunay triangulation
 
 voronoi [-t -p -d]  [filename]
@@ -96,7 +93,7 @@ On unsorted data uniformly distributed in the unit square, voronoi uses about
 AUTHOR
 Steve J. Fortune (1987) A Sweepline Algorithm for Voronoi Diagrams,
 Algorithmica 2, 153-174.
-"""
+""")
 
 #############################################################################
 #
@@ -123,14 +120,16 @@ Algorithmica 2, 153-174.
 #        Delaunay triangle.
 #
 #############################################################################
+
+
 import math
 import sys
 import getopt
 TOLERANCE = 1e-9
 BIG_FLOAT = 1e38
 
-#------------------------------------------------------------------
 
+# ------------------------------------------------------------------
 
 class Context(object):
 
@@ -156,39 +155,47 @@ class Context(object):
 
     def outSite(self, s):
         if self.debug:
-            print "site (%d) at %f %f" % (s.sitenum, s.x, s.y)
+            # fix_print_with_import
+            print("site (%d) at %f %f" % (s.sitenum, s.x, s.y))
         elif(self.triangulate):
             pass
         elif self.plot:
             self.circle(s.x, s.y, None)  # No radius?
         elif(self.doPrint):
-            print "s %f %f" % (s.x, s.y)
+            # fix_print_with_import
+            print("s %f %f" % (s.x, s.y))
 
     def outVertex(self, s):
         self.vertices.append((s.x, s.y))
         if(self.debug):
-            print "vertex(%d) at %f %f" % (s.sitenum, s.x, s.y)
+            # fix_print_with_import
+            print("vertex(%d) at %f %f" % (s.sitenum, s.x, s.y))
         elif(self.triangulate):
             pass
         elif(self.doPrint and not self.plot):
-            print "v %f %f" % (s.x, s.y)
+            # fix_print_with_import
+            print("v %f %f" % (s.x, s.y))
 
     def outTriple(self, s1, s2, s3):
         self.triangles.append((s1.sitenum, s2.sitenum, s3.sitenum))
         if(self.debug):
-            print "circle through left=%d right=%d bottom=%d" % (s1.sitenum, s2.sitenum, s3.sitenum)
+            # fix_print_with_import
+            print("circle through left=%d right=%d bottom=%d" % (s1.sitenum, s2.sitenum, s3.sitenum))
         elif(self.triangulate and self.doPrint and not self.plot):
-            print "%d %d %d" % (s1.sitenum, s2.sitenum, s3.sitenum)
+            # fix_print_with_import
+            print("%d %d %d" % (s1.sitenum, s2.sitenum, s3.sitenum))
 
     def outBisector(self, edge):
         self.lines.append((edge.a, edge.b, edge.c))
         if(self.debug):
-            print "line(%d) %gx+%gy=%g, bisecting %d %d" % (edge.edgenum, edge.a, edge.b, edge.c, edge.reg[0].sitenum, edge.reg[1].sitenum)
+            # fix_print_with_import
+            print("line(%d) %gx+%gy=%g, bisecting %d %d" % (edge.edgenum, edge.a, edge.b, edge.c, edge.reg[0].sitenum, edge.reg[1].sitenum))
         elif(self.triangulate):
             if(self.plot):
                 self.line(edge.reg[0].x, edge.reg[0].y, edge.reg[1].x, edge.reg[1].y)
         elif(self.doPrint and not self.plot):
-            print "l %f %f %f" % (edge.a, edge.b, edge.c)
+            # fix_print_with_import
+            print("l %f %f %f" % (edge.a, edge.b, edge.c))
 
     def outEdge(self, edge):
         sitenumL = -1
@@ -208,154 +215,149 @@ class Context(object):
             if self.plot:
                 self.clip_line(edge)
             elif(self.doPrint):
-                print "e %d" % edge.edgenum,
-                print " %d " % sitenumL,
-                print "%d" % sitenumR
+                # fix_print_with_import
+                print("e %d %d %d" % (edge.edgenum, sitenumL, sitenumR))
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 
 
 def voronoi(siteList, context):
-    try:
-        edgeList = EdgeList(siteList.xmin, siteList.xmax, len(siteList))
-        priorityQ = PriorityQueue(siteList.ymin, siteList.ymax, len(siteList))
-        siteIter = siteList.iterator()
+    edgeList = EdgeList(siteList.xmin, siteList.xmax, len(siteList))
+    priorityQ = PriorityQueue(siteList.ymin, siteList.ymax, len(siteList))
+    siteIter = siteList.iterator()
 
-        bottomsite = siteIter.next()
-        context.outSite(bottomsite)
-        newsite = siteIter.next()
-        minpt = Site(-BIG_FLOAT, -BIG_FLOAT)
-        while True:
-            if not priorityQ.isEmpty():
-                minpt = priorityQ.getMinPt()
+    bottomsite = next(siteIter)
+    context.outSite(bottomsite)
+    newsite = next(siteIter)
+    minpt = Site(-BIG_FLOAT, -BIG_FLOAT)
+    while True:
+        if not priorityQ.isEmpty():
+            minpt = priorityQ.getMinPt()
 
-            if (newsite and (priorityQ.isEmpty() or cmp(newsite, minpt) < 0)):
-                # newsite is smallest -  this is a site event
-                context.outSite(newsite)
+        if (newsite and (priorityQ.isEmpty() or cmp(newsite, minpt) < 0)):
+            # newsite is smallest -  this is a site event
+            context.outSite(newsite)
 
-                # get first Halfedge to the LEFT and RIGHT of the new site
-                lbnd = edgeList.leftbnd(newsite)
-                rbnd = lbnd.right
+            # get first Halfedge to the LEFT and RIGHT of the new site
+            lbnd = edgeList.leftbnd(newsite)
+            rbnd = lbnd.right
 
-                # if this halfedge has no edge, bot = bottom site (whatever that is)
-                # create a new edge that bisects
-                bot = lbnd.rightreg(bottomsite)
-                edge = Edge.bisect(bot, newsite)
-                context.outBisector(edge)
+            # if this halfedge has no edge, bot = bottom site (whatever that is)
+            # create a new edge that bisects
+            bot = lbnd.rightreg(bottomsite)
+            edge = Edge.bisect(bot, newsite)
+            context.outBisector(edge)
 
-                # create a new Halfedge, setting its pm field to 0 and insert
-                # this new bisector edge between the left and right vectors in
-                # a linked list
-                bisector = Halfedge(edge, Edge.LE)
-                edgeList.insert(lbnd, bisector)
+            # create a new Halfedge, setting its pm field to 0 and insert
+            # this new bisector edge between the left and right vectors in
+            # a linked list
+            bisector = Halfedge(edge, Edge.LE)
+            edgeList.insert(lbnd, bisector)
 
-                # if the new bisector intersects with the left edge, remove
-                # the left edge's vertex, and put in the new one
-                p = lbnd.intersect(bisector)
-                if p is not None:
-                    priorityQ.delete(lbnd)
-                    priorityQ.insert(lbnd, p, newsite.distance(p))
+            # if the new bisector intersects with the left edge, remove
+            # the left edge's vertex, and put in the new one
+            p = lbnd.intersect(bisector)
+            if p is not None:
+                priorityQ.delete(lbnd)
+                priorityQ.insert(lbnd, p, newsite.distance(p))
 
-                # create a new Halfedge, setting its pm field to 1
-                # insert the new Halfedge to the right of the original bisector
-                lbnd = bisector
-                bisector = Halfedge(edge, Edge.RE)
-                edgeList.insert(lbnd, bisector)
+            # create a new Halfedge, setting its pm field to 1
+            # insert the new Halfedge to the right of the original bisector
+            lbnd = bisector
+            bisector = Halfedge(edge, Edge.RE)
+            edgeList.insert(lbnd, bisector)
 
-                # if this new bisector intersects with the right Halfedge
-                p = bisector.intersect(rbnd)
-                if p is not None:
-                    # push the Halfedge into the ordered linked list of vertices
-                    priorityQ.insert(bisector, p, newsite.distance(p))
+            # if this new bisector intersects with the right Halfedge
+            p = bisector.intersect(rbnd)
+            if p is not None:
+                # push the Halfedge into the ordered linked list of vertices
+                priorityQ.insert(bisector, p, newsite.distance(p))
 
-                newsite = siteIter.next()
+            newsite = next(siteIter)
 
-            elif not priorityQ.isEmpty():
-                # intersection is smallest - this is a vector (circle) event
+        elif not priorityQ.isEmpty():
+            # intersection is smallest - this is a vector (circle) event
 
-                # pop the Halfedge with the lowest vector off the ordered list of
-                # vectors.  Get the Halfedge to the left and right of the above HE
-                # and also the Halfedge to the right of the right HE
-                lbnd = priorityQ.popMinHalfedge()
-                llbnd = lbnd.left
-                rbnd = lbnd.right
-                rrbnd = rbnd.right
+            # pop the Halfedge with the lowest vector off the ordered list of
+            # vectors.  Get the Halfedge to the left and right of the above HE
+            # and also the Halfedge to the right of the right HE
+            lbnd = priorityQ.popMinHalfedge()
+            llbnd = lbnd.left
+            rbnd = lbnd.right
+            rrbnd = rbnd.right
 
-                # get the Site to the left of the left HE and to the right of
-                # the right HE which it bisects
-                bot = lbnd.leftreg(bottomsite)
-                top = rbnd.rightreg(bottomsite)
+            # get the Site to the left of the left HE and to the right of
+            # the right HE which it bisects
+            bot = lbnd.leftreg(bottomsite)
+            top = rbnd.rightreg(bottomsite)
 
-                # output the triple of sites, stating that a circle goes through them
-                mid = lbnd.rightreg(bottomsite)
-                context.outTriple(bot, top, mid)
+            # output the triple of sites, stating that a circle goes through them
+            mid = lbnd.rightreg(bottomsite)
+            context.outTriple(bot, top, mid)
 
-                # get the vertex that caused this event and set the vertex number
-                # couldn't do this earlier since we didn't know when it would be processed
-                v = lbnd.vertex
-                siteList.setSiteNumber(v)
-                context.outVertex(v)
+            # get the vertex that caused this event and set the vertex number
+            # couldn't do this earlier since we didn't know when it would be processed
+            v = lbnd.vertex
+            siteList.setSiteNumber(v)
+            context.outVertex(v)
 
-                # set the endpoint of the left and right Halfedge to be this vector
-                if lbnd.edge.setEndpoint(lbnd.pm, v):
-                    context.outEdge(lbnd.edge)
+            # set the endpoint of the left and right Halfedge to be this vector
+            if lbnd.edge.setEndpoint(lbnd.pm, v):
+                context.outEdge(lbnd.edge)
 
-                if rbnd.edge.setEndpoint(rbnd.pm, v):
-                    context.outEdge(rbnd.edge)
+            if rbnd.edge.setEndpoint(rbnd.pm, v):
+                context.outEdge(rbnd.edge)
 
-                # delete the lowest HE, remove all vertex events to do with the
-                # right HE and delete the right HE
-                edgeList.delete(lbnd)
-                priorityQ.delete(rbnd)
-                edgeList.delete(rbnd)
+            # delete the lowest HE, remove all vertex events to do with the
+            # right HE and delete the right HE
+            edgeList.delete(lbnd)
+            priorityQ.delete(rbnd)
+            edgeList.delete(rbnd)
 
-                # if the site to the left of the event is higher than the Site
-                # to the right of it, then swap them and set 'pm' to RIGHT
-                pm = Edge.LE
-                if bot.y > top.y:
-                    bot, top = top, bot
-                    pm = Edge.RE
+            # if the site to the left of the event is higher than the Site
+            # to the right of it, then swap them and set 'pm' to RIGHT
+            pm = Edge.LE
+            if bot.y > top.y:
+                bot, top = top, bot
+                pm = Edge.RE
 
-                # Create an Edge (or line) that is between the two Sites.  This
-                # creates the formula of the line, and assigns a line number to it
-                edge = Edge.bisect(bot, top)
-                context.outBisector(edge)
+            # Create an Edge (or line) that is between the two Sites.  This
+            # creates the formula of the line, and assigns a line number to it
+            edge = Edge.bisect(bot, top)
+            context.outBisector(edge)
 
-                # create a HE from the edge
-                bisector = Halfedge(edge, pm)
+            # create a HE from the edge
+            bisector = Halfedge(edge, pm)
 
-                # insert the new bisector to the right of the left HE
-                # set one endpoint to the new edge to be the vector point 'v'
-                # If the site to the left of this bisector is higher than the right
-                # Site, then this endpoint is put in position 0; otherwise in pos 1
-                edgeList.insert(llbnd, bisector)
-                if edge.setEndpoint(Edge.RE - pm, v):
-                    context.outEdge(edge)
+            # insert the new bisector to the right of the left HE
+            # set one endpoint to the new edge to be the vector point 'v'
+            # If the site to the left of this bisector is higher than the right
+            # Site, then this endpoint is put in position 0; otherwise in pos 1
+            edgeList.insert(llbnd, bisector)
+            if edge.setEndpoint(Edge.RE - pm, v):
+                context.outEdge(edge)
 
-                # if left HE and the new bisector don't intersect, then delete
-                # the left HE, and reinsert it
-                p = llbnd.intersect(bisector)
-                if p is not None:
-                    priorityQ.delete(llbnd)
-                    priorityQ.insert(llbnd, p, bot.distance(p))
+            # if left HE and the new bisector don't intersect, then delete
+            # the left HE, and reinsert it
+            p = llbnd.intersect(bisector)
+            if p is not None:
+                priorityQ.delete(llbnd)
+                priorityQ.insert(llbnd, p, bot.distance(p))
 
-                # if right HE and the new bisector don't intersect, then reinsert it
-                p = bisector.intersect(rrbnd)
-                if p is not None:
-                    priorityQ.insert(bisector, p, bot.distance(p))
-            else:
-                break
+            # if right HE and the new bisector don't intersect, then reinsert it
+            p = bisector.intersect(rrbnd)
+            if p is not None:
+                priorityQ.insert(bisector, p, bot.distance(p))
+        else:
+            break
 
-        he = edgeList.leftend.right
-        while he is not edgeList.rightend:
-            context.outEdge(he.edge)
-            he = he.right
-        Edge.EDGE_NUM = 0
-    except Exception as err:
-        print "######################################################"
-        print unicode(err)
+    he = edgeList.leftend.right
+    while he is not edgeList.rightend:
+        context.outEdge(he.edge)
+        he = he.right
+    Edge.EDGE_NUM = 0
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 
 
 def isEqual(a, b, relativeError=TOLERANCE):
@@ -363,7 +365,7 @@ def isEqual(a, b, relativeError=TOLERANCE):
     norm = max(abs(a), abs(b))
     return (norm < relativeError) or (abs(a - b) < (relativeError * norm))
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 
 
 class Site(object):
@@ -374,26 +376,28 @@ class Site(object):
         self.sitenum = sitenum
 
     def dump(self):
-        print "Site #%d (%g, %g)" % (self.sitenum, self.x, self.y)
+        # fix_print_with_import
+        print("Site #%d (%g, %g)" % (self.sitenum, self.x, self.y))
 
-    def __cmp__(self, other):
+    def __eq__(self, other):
+        return (self.x == other.x) and (self.y == other.y)
+
+    def __lt__(self, other):
         if self.y < other.y:
-            return -1
+            return True
         elif self.y > other.y:
-            return 1
+            return False
         elif self.x < other.x:
-            return -1
-        elif self.x > other.x:
-            return 1
+            return True
         else:
-            return 0
+            return False
 
     def distance(self, other):
         dx = self.x - other.x
         dy = self.y - other.y
         return math.sqrt(dx * dx + dy * dy)
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 
 
 class Edge(object):
@@ -411,9 +415,12 @@ class Edge(object):
         self.edgenum = 0
 
     def dump(self):
-        print "(#%d a=%g, b=%g, c=%g)" % (self.edgenum, self.a, self.b, self.c)
-        print "ep", self.ep
-        print "reg", self.reg
+        # fix_print_with_import
+        print("(#%d a=%g, b=%g, c=%g)" % (self.edgenum, self.a, self.b, self.c))
+        # fix_print_with_import
+        print("ep", self.ep)
+        # fix_print_with_import
+        print("reg", self.reg)
 
     def setEndpoint(self, lrFlag, site):
         self.ep[lrFlag] = site
@@ -424,7 +431,7 @@ class Edge(object):
     @staticmethod
     def bisect(s1, s2):
         newedge = Edge()
-        newedge.reg[0] = s1 # store the sites that this edge is bisecting
+        newedge.reg[0] = s1  # store the sites that this edge is bisecting
         newedge.reg[1] = s2
 
         # to begin with, there are no endpoints on the bisector - it goes to infinity
@@ -454,7 +461,7 @@ class Edge(object):
         return newedge
 
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 class Halfedge(object):
 
     def __init__(self, edge=None, pm=Edge.LE):
@@ -467,29 +474,38 @@ class Halfedge(object):
         self.ystar = BIG_FLOAT
 
     def dump(self):
-        print "Halfedge--------------------------"
-        print "left: ", self.left
-        print "right: ", self.right
-        print "edge: ", self.edge
-        print "pm: ", self.pm
-        print "vertex: ",
+        # fix_print_with_import
+        print("Halfedge--------------------------")
+        # fix_print_with_import
+        print("left: ", self.left)
+        # fix_print_with_import
+        print("right: ", self.right)
+        # fix_print_with_import
+        print("edge: ", self.edge)
+        # fix_print_with_import
+        print("pm: ", self.pm)
+        # fix_print_with_import
+        print("vertex:")
         if self.vertex:
             self.vertex.dump()
         else:
-            print "None"
-        print "ystar: ", self.ystar
+            # fix_print_with_import
+            print("None")
+        # fix_print_with_import
+        print("ystar: ", self.ystar)
 
-    def __cmp__(self, other):
-        if self.ystar > other.ystar:
-            return 1
-        elif self.ystar < other.ystar:
-            return -1
-        elif self.vertex.x > other.vertex.x:
-            return 1
+    def __eq__(self, other):
+        return (self.vertex.x == other.vertex.x) and (self.ystar == other.ystar)
+
+    def __lt__(self, other):
+        if self.ystar < other.ystar:
+            return True
+        elif self.ystar > other.ystar:
+            return False
         elif self.vertex.x < other.vertex.x:
-            return -1
+            return True
         else:
-            return 0
+            return False
 
     def leftreg(self, default):
         if not self.edge:
@@ -549,7 +565,7 @@ class Halfedge(object):
         else:
             return not above
 
-    #--------------------------
+    # --------------------------
     # create a new site where the Halfedges el1 and el2 intersect
     def intersect(self, other):
         e1 = self.edge
@@ -584,7 +600,7 @@ class Halfedge(object):
         return Site(xint, yint)
 
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 class EdgeList(object):
 
     def __init__(self, xmin, xmax, nsites):
@@ -648,7 +664,7 @@ class EdgeList(object):
                     break
                 i += 1
 
-        # Now search linear list of halfedges for the corect one
+        # Now search linear list of halfedges for the correct one
         if (he is self.leftend) or (he is not self.rightend and he.isPointRightOf(pt)):
             he = he.right
             while he is not self.rightend and he.isPointRightOf(pt):
@@ -665,7 +681,7 @@ class EdgeList(object):
         return he
 
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 class PriorityQueue(object):
 
     def __init__(self, ymin, ymax, nsites):
@@ -730,7 +746,7 @@ class PriorityQueue(object):
         return curr
 
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 class SiteList(object):
 
     def __init__(self, pointList):
@@ -759,13 +775,15 @@ class SiteList(object):
 
     class Iterator(object):
 
-        def __init__(this, lst): this.generator = (s for s in lst)
+        def __init__(this, lst):
+            this.generator = (s for s in lst)
 
-        def __iter__(this): return this
+        def __iter__(this):
+            return this
 
-        def next(this):
+        def __next__(this):
             try:
-                return this.generator.next()
+                return next(this.generator)
             except StopIteration:
                 return None
 
@@ -778,13 +796,17 @@ class SiteList(object):
     def __len__(self):
         return len(self.__sites)
 
-    def _getxmin(self): return self.__xmin
+    def _getxmin(self):
+        return self.__xmin
 
-    def _getymin(self): return self.__ymin
+    def _getymin(self):
+        return self.__ymin
 
-    def _getxmax(self): return self.__xmax
+    def _getxmax(self):
+        return self.__xmax
 
-    def _getymax(self): return self.__ymax
+    def _getymax(self):
+        return self.__ymax
 
     xmin = property(_getxmin)
     ymin = property(_getymin)
@@ -792,7 +814,7 @@ class SiteList(object):
     ymax = property(_getymax)
 
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 def computeVoronoiDiagram(points):
     """ Takes a list of point objects (which must have x and y fields).
         Returns a 3-tuple of:
@@ -811,7 +833,7 @@ def computeVoronoiDiagram(points):
     voronoi(siteList, context)
     return (context.vertices, context.lines, context.edges)
 
-#------------------------------------------------------------------
+# ------------------------------------------------------------------
 
 
 def computeDelaunayTriangulation(points):
@@ -825,7 +847,8 @@ def computeDelaunayTriangulation(points):
     voronoi(siteList, context)
     return context.triangles
 
-#-----------------------------------------------------------------------------
+
+# -----------------------------------------------------------------------------
 if __name__ == "__main__":
     try:
         optlist, args = getopt.getopt(sys.argv[1:], "thdp")
@@ -865,3 +888,13 @@ if __name__ == "__main__":
 
     sl = SiteList(pts)
     voronoi(sl, c)
+
+
+def cmp(a, b):
+    """Compare the two objects x and y and return an integer according to the
+    outcome. The return value is negative if x < y, zero if x == y and strictly
+    positive if x > y.
+
+    In python 2 cmp() was a built in function but in python 3 is gone.
+    """
+    return (b < a) - (a < b)
